-------------------------------------------------------------------------------
-- Title      : Adressed Message Decoder Module
-- Project    : RCU Trigger Receiver
-------------------------------------------------------------------------------
-- File             : $RCSfile: addressed_msg_decoder.vhd,v $
-- Last edited by   : $Author: alme $
-- Last update      : $Date: 2009-10-19 05:13:29 $
-- Current Revision : $Revision: 1.10 $
-------------------------------------------------------------------------------
-- Description: Decodes individually-addressed commands from serial_b_channel
--              into messages. Generates L2a and L2r signal.
-------------------------------------------------------------------------------
-- Revision History : 
-- http://www.ift.uib.no/kjekscgi-bin/viewcvs.cgi/vhdlcvs/trigger_receiver/
-------------------------------------------------------------------------------
--
-- This file is property of and copyright by the Instrumentation and 
-- Electronics Section, Dep. of Physics and Technology
-- University of Bergen, Norway, 2005
-- This file has been written by Johan Alme
-- Johan.Alme@ift.uib.no
--
-- Permission to use, copy, modify and distribute this firmware and its  
-- documentation strictly for non-commercial purposes is hereby granted  
-- without fee, provided that the above copyright notice appears in all  
-- copies and that both the copyright notice and this permission notice  
-- appear in the supporting documentation. The authors make no claims    
-- about the suitability of this firmware for any purpose. It is         
-- provided "as is" without express or implied warranty.                 
--
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

library work;
use work.trigger_receiver_pkg.all;

entity addressed_message_decoder is
  port(
    clk            : in  std_logic;
    reset_n        : in  std_logic;
    module_reset   : in  std_logic;
    
    adrdata        : in  std_logic_vector(31 downto 0);
    adrdata_ready  : in  std_logic;
    -- used to reset messages.
    init_sequence  : in  std_logic;
    L2_trigger     : in  std_logic; -- signalises when reception is finished/timeout
    L1a_msg_mask   : in  std_logic;
    error_reg      : out std_logic_vector(4 downto 0);
    msg_error      : out std_logic; -- used for counting
    receiving_L1   : out std_logic;
    receiving_L2a  : out std_logic;
    receiving_L2r  : out std_logic;
    FEE_reset      : out std_logic;
    --L1_message     : out message_array(0 to 4);--don't store L1class - it is not used
    L1_message     : out message_array(0 to 0);
    L2a_message    : out message_array(0 to 7);
    L2r_message    : out message_array(0 to 0)
  );
end addressed_message_decoder;

architecture arc of addressed_message_decoder is
  
  type state is (s_idle,
                 s_chk_address,
                 s_L1_data1,
                 s_L1_data2,
                 s_L1_data3,
                 s_L1_data4,
                 s_L2a_data1,
                 s_L2a_data2,
                 s_L2a_data3,
                 s_L2a_data4,
                 s_L2a_data5,
                 s_L2a_data6,
                 s_L2a_data7,
                 s_FEE_reset);
                 
                
  signal current_state, next_state: state;  
  -- used to remember last word receieved in the states
  signal L1_state, L2_state: state; --, RoI_state : state;
  
  --signal L1_message_reg     : message_array(0 to 4);
  signal L1_message_reg     : message_array(0 to 0);
  signal L2a_message_reg    : message_array(0 to 7);
	signal L2r_message_reg    : message_array(0 to 0);
	
  -- general signals and types
  signal receiving_L1_i      : std_logic := '0';
  signal receiving_L2a_i     : std_logic := '0';
  signal receiving_L2r_i     : std_logic := '0';
  
  signal TTCrx_address       : std_logic_vector(15 downto 0); -- adrdata(31 downto 16);
  signal sub_address         : std_logic_vector( 3 downto 0); -- adrdata(15 downto 12);
  signal data                : std_logic_vector(11 downto 0); -- adrdata(11 downto 0);
  
  signal unknown_msg_address   : std_logic :='0';-- error_reg(0);
  signal Incomplete_L1_msg     : std_logic :='0';-- error_reg(1);
  signal Incomplete_L2a_msg    : std_logic :='0';-- error_reg(2);
  signal TTCrx_addr_error      : std_logic :='0';-- error_reg(4);
  
begin
-- set internal signals
TTCrx_address      <= adrdata(31 downto 16);
sub_address        <= adrdata(15 downto 12);
data               <= adrdata(11 downto 0);
 


-- set outputs
receiving_L1   <= receiving_L1_i;
receiving_L2a  <= receiving_L2a_i;
receiving_L2r  <= receiving_L2r_i;
L1_message     <= L1_message_reg;
L2a_message    <= L2a_message_reg;
L2r_message    <= L2r_message_reg;
--masking the L1 message errors if chosen not to retrieve L1a messages
error_reg      <= TTCrx_addr_error & '0' & Incomplete_L2a_msg & (Incomplete_L1_msg and L1a_msg_mask) & unknown_msg_address;

p_state_driver : process(clk, reset_n)
begin
  if (reset_n = '0') then
    current_state <= s_idle;
  elsif rising_edge(clk) then
    if (module_reset = '1') then
      current_state <= s_idle;
    else
      current_state <= next_state;
    end if;
  end if;
end process p_state_driver;

p_decoder: process(current_state, adrdata_ready, L1_state, L2_state, sub_address)--, RoI_state, L2_state, sub_address)
begin 
  case current_state is
    when s_idle =>
      if (adrdata_ready = '1') then
        if (sub_address = c_L1_header) then -- and TTCrx_address = c_TTCrx_addr) then
          next_state <= s_L1_data1;
        elsif (sub_address = c_L2a_header) then -- and TTCrx_address = c_TTCrx_addr) then
          next_state <= s_L2a_data1;
        elsif (sub_address = c_L2r_header) then -- and TTCrx_address = c_TTCrx_addr) then
          next_state <= s_idle; -- just one word to receive
        else -- set error flags and do nothing  
          next_state <= s_idle;
        end if;
      else
        next_state <= s_idle;
      end if;
    
    -- L1 receiver  
    when s_L1_data1 =>
      if (adrdata_ready = '1') then    -- new data
        if (sub_address = c_L1_data) then -- and TTCrx_address = c_TTCrx_addr) then -- should be L1 data
          next_state <= s_L1_data2;
        else
          next_state <= s_chk_address;
        end if;
      else
        next_state <= s_L1_data1;
      end if;
    when s_L1_data2 =>
      if (adrdata_ready = '1') then    -- new data
        if (sub_address = c_L1_data) then -- and TTCrx_address = c_TTCrx_addr) then -- should be L1 data
          next_state <= s_L1_data3;
        else
          next_state <= s_chk_address;
        end if;
      else
        next_state <= s_L1_data2;
      end if;
    when s_L1_data3 =>
      if (adrdata_ready = '1') then    -- new data
        if (sub_address = c_L1_data) then -- and TTCrx_address = c_TTCrx_addr) then -- should be L1 data
          next_state <= s_L1_data4;
        else
          next_state <= s_chk_address;
        end if;
      else
        next_state <= s_L1_data3;
      end if;
    when s_L1_data4 =>
      if (adrdata_ready = '1') then    -- new data
        if (sub_address = c_L1_data) then -- and TTCrx_address = c_TTCrx_addr) then -- should be L1 data
          next_state <= s_idle;
        else
          next_state <= s_chk_address;
        end if;
      else
        next_state <= s_L1_data4;
      end if;
      
    -- L2a receiver  
    when s_L2a_data1 =>
      if (adrdata_ready = '1') then    -- new data
        if (sub_address = c_L2a_data) then -- and TTCrx_address = c_TTCrx_addr) then -- should be L2a data
          next_state <= s_L2a_data2;
        else
          next_state <= s_chk_address;
        end if;
      else
        next_state <= s_L2a_data1;
      end if;
    when s_L2a_data2 =>
      if (adrdata_ready = '1') then    -- new data
        if (sub_address = c_L2a_data) then --and TTCrx_address = c_TTCrx_addr) then -- should be L2a data
          next_state <= s_L2a_data3;
        else
          next_state <= s_chk_address;
        end if;
      else
        next_state <= s_L2a_data2;
      end if;
    when s_L2a_data3 =>
      if (adrdata_ready = '1') then    -- new data
        if (sub_address = c_L2a_data) then --and TTCrx_address = c_TTCrx_addr) then -- should be L2a data
          next_state <= s_L2a_data4;
        else
          next_state <= s_chk_address;
        end if;
      else
        next_state <= s_L2a_data3;
      end if;
    when s_L2a_data4 =>
      if (adrdata_ready = '1') then    -- new data
        if (sub_address = c_L2a_data) then --and TTCrx_address = c_TTCrx_addr) then -- should be L2a data
          next_state <= s_L2a_data5;
        else
          next_state <= s_chk_address;
        end if;
      else
        next_state <= s_L2a_data4;
      end if;
    when s_L2a_data5 =>
      if (adrdata_ready = '1') then     -- new data
        if (sub_address = c_L2a_data) then -- and TTCrx_address = c_TTCrx_addr) then -- should be L2a data
          next_state <= s_L2a_data6;
        else                             --  
          next_state <= s_chk_address;
        end if;
      else
        next_state <= s_L2a_data5;
      end if;
    when s_L2a_data6 =>
      if (adrdata_ready = '1') then    -- new data
        if (sub_address = c_L2a_data) then -- and TTCrx_address = c_TTCrx_addr) then -- should be L2a data
          next_state <= s_L2a_data7;
        else
          next_state <= s_chk_address;
        end if;
      else
        next_state <= s_L2a_data6;
      end if;
    when s_L2a_data7 =>
      if (adrdata_ready = '1') then    -- new data
        if (sub_address = c_L2a_data) then -- and TTCrx_address = c_TTCrx_addr) then -- should be L2a data
          next_state <= s_idle;
        else
          next_state <= s_chk_address;
        end if;
      else
        next_state <= s_L2a_data7;
      end if;
      
    when s_chk_address =>
      -- If this state is entered, there's an error in the transmission
      -- Either missing words or an unknown message address.
      -- If a header word is found, an attempt to receive the the message is done
      -- If there's a data word an error is reported and is back to idle.
      if (sub_address = c_L1_header) then
        next_state <= s_L1_data1;
      elsif (sub_address = c_L2a_header) then
        next_state <= s_L2a_data1;
      elsif (sub_address = c_L2r_header) then
        next_state <= s_idle;
      elsif (sub_address = c_L1_data) then
        next_state <= L1_state;
      elsif (sub_address = c_L2a_data) then
        next_state <= L2_state;
      else
        next_state <= s_idle;
      end if;
      
    when others =>
      next_state <= s_idle;
      
  end case;    
end process p_decoder;

-- purpose: Assigning the outputs of the state machine.
p_assign_outputs : process(clk, reset_n)
begin
  if (reset_n = '0') then
    FEE_reset             <= '0';
    L1_message_reg(0)  <= (others => '0');
    for i in 0 to 7 loop
      L2a_message_reg(i) <= (others => '0');
    end loop;
    L2r_message_reg(0) <= (others => '0');
    receiving_L1_i  <= '0';
    receiving_L2a_i <= '0';
    receiving_L2r_i <= '0';
    --receiving_RoI_i <= '0';
    unknown_msg_address   <= '0';
    Incomplete_L1_msg     <= '0';
    Incomplete_L2a_msg    <= '0';   
    TTCrx_addr_error      <= '0';
    msg_error             <= '0';
    L1_state             <= s_idle;
    L2_state             <= s_idle;
      
  elsif rising_edge(clk) then
    -- default
    FEE_reset       <= '0';
    --comment the defaults out - since one can receive many things at the same time
    msg_error       <= '0';
    -- pulsing message error on the end of a sequence in case of incomplete messages
    if (L2_trigger = '1' and (Incomplete_L1_msg = '1' or Incomplete_L2a_msg = '1')) then -- or Incomplete_RoI_msg = '1')) then
      msg_error          <= '1';
    end if;
    if (adrdata_ready = '1' and TTCrx_address /= c_TTCrx_addr) then
      TTCrx_addr_error   <= '1';
      msg_error          <= '1';
    end if;
          
    -- reset registers when new sequence starts. (For all states)
    -- also clears errors
    if (init_sequence = '1' or module_reset = '1') then
      L1_state             <= s_idle;
      L2_state             <= s_idle;
      
      unknown_msg_address   <= '0';
      Incomplete_L1_msg     <= '0';
      Incomplete_L2a_msg    <= '0';
      receiving_L1_i        <= '0';
      receiving_L2a_i       <= '0';
      receiving_L2r_i       <= '0';
      TTCrx_addr_error      <= '0';
      --for i in 0 to 4 loop
      --  L1_message_reg(i)   <= (others => '0');
      --end loop;
      L1_message_reg(0)  <= (others => '0');
      for i in 0 to 7 loop
        L2a_message_reg(i)  <= (others => '0');
      end loop;
      L2r_message_reg(0)    <= (others => '0');
    end if;

    case current_state is
      when s_idle =>
        L1_state             <= s_idle;
        L2_state             <= s_idle;
        receiving_L1_i       <= '0';
        receiving_L2a_i      <= '0';
        receiving_L2r_i      <= '0';
        
        -- new data arrives
        if (adrdata_ready = '1') then
          case sub_address is
            when c_L1_header =>
              L1_message_reg(0) <= data;
              Incomplete_L1_msg <= '1';
              receiving_L1_i    <= '1';
            when c_L2a_header =>
              L2a_message_reg(0) <= data;
              Incomplete_L2a_msg <= '1';
              receiving_L2a_i    <= '1';
            when c_L2r_header =>
              L2r_message_reg(0) <= data;
              receiving_L2r_i    <= '1';
            when c_FEE_reset => 
              FEE_reset          <= '1';
            when c_L1_data =>
              -- do nothing
            when c_L2a_data =>
              --do nothing
            when others =>
                -- unknown message address
              unknown_msg_address <= '1';
              msg_error           <= '1';
          end case;
        end if;

      -- L1 receiver
      when s_L1_data1 =>
        L1_state          <= s_L1_data1;
        Incomplete_L1_msg <= '1';
        receiving_L1_i    <= '1';
      when s_L1_data2 =>
        L1_state          <= s_L1_data2;
        Incomplete_L1_msg <= '1';
        receiving_L1_i    <= '1';
      when s_L1_data3 =>
        L1_state          <= s_L1_data3;
        Incomplete_L1_msg <= '1';
        receiving_L1_i    <= '1';
      when s_L1_data4 =>
        L1_state          <= s_L1_data4;
        receiving_L1_i    <= '1';
        if (adrdata_ready = '1') then    -- new data
          if (sub_address = c_L1_data) then -- should be L2a data
            Incomplete_L1_msg <= '0'; -- message completely received.   
          end if;
        end if;
        
      -- L2a receiver
      when s_L2a_data1 =>
        L2_state           <= s_L2a_data1;
        Incomplete_L2a_msg <= '1';   
        receiving_L2a_i    <= '1';
        if (adrdata_ready = '1') then    -- new data
          if (sub_address = c_L2a_data) then -- should be L2a data
            L2a_message_reg(1)  <= data;
          end if;
        end if;
      when s_L2a_data2 =>
        L2_state           <= s_L2a_data2;
        Incomplete_L2a_msg <= '1';   
        receiving_L2a_i    <= '1';
        if (adrdata_ready = '1') then    -- new data
          if (sub_address = c_L2a_data) then -- should be L2a data
            L2a_message_reg(2)  <= data;
          end if;
        end if;
      when s_L2a_data3 =>
        L2_state           <= s_L2a_data3;
        Incomplete_L2a_msg <= '1';   
        receiving_L2a_i    <= '1';
        if (adrdata_ready = '1') then    -- new data
          if (sub_address = c_L2a_data) then -- should be L2a data
            L2a_message_reg(3)  <= data;
          end if;
        end if;
      when s_L2a_data4 =>
        L2_state           <= s_L2a_data4;
        Incomplete_L2a_msg <= '1';   
        receiving_L2a_i    <= '1';
        if (adrdata_ready = '1') then    -- new data
          if (sub_address = c_L2a_data) then -- should be L2a data
            L2a_message_reg(4)  <= data;
          end if;
        end if;
      when s_L2a_data5 =>
        L2_state           <= s_L2a_data5;
        Incomplete_L2a_msg <= '1';   
        receiving_L2a_i    <= '1';
        if (adrdata_ready = '1') then     -- new data
          if (sub_address = c_L2a_data) then -- should be L2a data
            L2a_message_reg(5)  <= data;
          end if;
        end if;
      when s_L2a_data6 =>
        L2_state           <= s_L2a_data6;
        Incomplete_L2a_msg <= '1';   
        receiving_L2a_i    <= '1';
        if (adrdata_ready = '1') then    -- new data
          if (sub_address = c_L2a_data) then -- should be L2a data
            L2a_message_reg(6)  <= data;
          end if;
        end if;
      when s_L2a_data7 =>
        L2_state           <= s_L2a_data7;
        receiving_L2a_i <= '1';
        if (adrdata_ready = '1') then    -- new data
          if (sub_address = c_L2a_data) then -- should be L2a data
            L2a_message_reg(7) <= data;
            Incomplete_L2a_msg <= '0'; -- message completely received.   
           end if;
        end if;
       
      when s_chk_address =>
        -- check what is received. If it is a header continue to decode.
        case sub_address is
          when c_L1_header =>
            L1_message_reg(0)  <= data;
            Incomplete_L1_msg  <= '1';   
            receiving_L1_i     <= '1';
          when c_L2a_header =>
            L2a_message_reg(0) <= data;
            Incomplete_L2a_msg <= '1';   
            receiving_L2a_i    <= '1';
          when c_L2r_header =>
            L2r_message_reg(0) <= data;
            receiving_L2r_i    <= '1';
          when c_FEE_reset =>
            FEE_reset          <= '1';
          when c_L1_data =>
            -- do nothing
          when c_L2a_data =>
            -- do nothing
          when others =>
            -- unknown message address           
            unknown_msg_address   <= '1';
            msg_error             <= '1';
        end case;
      when others =>
        -- do nothing
    end case;    
  end if;
end process p_assign_outputs;

end arc;
