-- Based on patent 4746898 by HP
-- By Arild Velure
-- 3/4 clock and data delay

library ieee;
use ieee.std_logic_1164.all;
ENTITY hp_cdr IS
   GENERIC (
      U_dly                          :  integer := 2);    
   PORT (
      rst                     : IN std_logic;   --  asynchronous reset.
      data_in                 : IN std_logic;   -- Differential Manchester encoded data stream.
      data_in_del             : IN std_logic;   -- Differential Manchester encoded data stream delayed by 3/4
      rx_ena                  : IN std_logic;   -- when active,begian decode
      err_det                 : OUT std_logic;   -- when active,it indicates the recovered data is incorrect.
      clk_out                 : OUT std_logic;   -- clock recovered from the Manchester encoded data stream.
      data_out                : OUT std_logic);   --  original data recovered from the Differential Manchester encoded data stream .
END ENTITY hp_cdr;

ARCHITECTURE hp_fig2 OF hp_cdr IS
  signal input_d:std_logic;
  signal output_e:std_logic;
  signal output_e_del:std_logic;
  signal output_f:std_logic;
  signal clk_out_reg:std_logic;
BEGIN
  PROCESS (clk_out_reg, rst)
   BEGIN
      IF (rst = '1') THEN
         data_out <= '0';
         err_det <= '0';   
      ELSIF (clk_out_reg'EVENT AND clk_out_reg = '1') THEN
         IF (rx_ena = '1') THEN
            data_out <= input_d; 
         END IF;
      END IF;
   END PROCESS;
   
   PROCESS (clk_out_reg, rst)
   BEGIN
      IF (rst = '1') THEN
        output_e <= '0';
        output_f <= '1';
      ELSIF (clk_out_reg'EVENT AND clk_out_reg = '1') THEN
         IF (rx_ena = '1') THEN
            output_e <= data_in; 
            output_f <= NOT data_in; 
         END IF;
      END IF;
   END PROCESS;
   
   PROCESS (input_d, rst) -- delays the output_e until a falling edge on input_d, essentially the next period on the input, avoids spikes on clock
   BEGIN
      IF (rst = '1') THEN
        output_e_del <= '0';
      ELSIF (input_d'EVENT AND input_d = '0') THEN
         IF (rx_ena = '1') THEN
            output_e_del <= output_e; 
         END IF;
      END IF;
   END PROCESS;
   
   input_d <= data_in XOR output_f;
   clk_out_reg <= output_e_del XOR data_in_del;
   clk_out <= output_e_del XOR data_in_del;
   
   END ARCHITECTURE hp_fig2;
   
ARCHITECTURE hp_fig4 OF hp_cdr IS
  signal input_d:std_logic;
  signal output_e:std_logic;
  signal output_e_del:std_logic;
  signal output_f:std_logic;
  signal clk_out_reg:std_logic;
BEGIN
  PROCESS (clk_out_reg, rst)
   BEGIN
      IF (rst = '1') THEN
         data_out <= '0';
         err_det <= '0';   
      ELSIF (clk_out_reg'EVENT AND clk_out_reg = '1') THEN
         IF (rx_ena = '1') THEN
            data_out <= input_d;  
         END IF;
      END IF;
   END PROCESS;
   
   PROCESS (clk_out_reg, rst)
   BEGIN
      IF (rst = '1') THEN
        output_e <= '0';
        output_f <= '1';
      ELSIF (clk_out_reg'EVENT AND clk_out_reg = '1') THEN
         IF (rx_ena = '1') THEN
            output_e <= data_in; -- after 2*xor delay
            output_f <= NOT data_in; -- after 2*xor delay
         END IF;
      END IF;
   END PROCESS;
   output_e_del <= transport output_e after 6.25 ns; -- defines pulsewidth of the short pulse
   input_d <= data_in XOR output_f; -- data in after xor delay
   clk_out_reg <= output_e_del XOR data_in_del;
   clk_out <= clk_out_reg; -- after xor delay
   
END ARCHITECTURE hp_fig4;
   
--ARCHITECTURE logical OF hp_cdr IS
--  signal clk_out_reg:std_logic;
--  signal data_out_reg:std_logic;
--  signal data_in_xor:std_logic;
--  signal data_in_xor_del:std_logic;
--  signal invert:std_logic;
--  signal clk_zero:std_logic;
--BEGIN
--  PROCESS (clk_out_reg, rst)
--   BEGIN
--      IF (rst = '1') THEN
--         data_out_reg <= '0';
--         err_det <= '0';   
--         invert <= '0';
--         clk_zero <= '0';
--      ELSIF (clk_out_reg'EVENT AND clk_out_reg = '1') THEN
--         IF (rx_ena = '1') THEN
--            data_out_reg <= data_in_xor_del; -- A change between second period and first equals a 1
--            if ((data_in_xor_del) = '0') then -- if data value is zero, then invert output clock based on current state of manchester signal
--              invert <= data_in_del;
--            else
--              invert <= invert;
--            end if;
--            clk_zero <= not(data_in xor data_in_del); -- If data value is zero, then set clock to zero
--         END IF;
--      ELSIF (clk_out_reg'EVENT AND clk_out_reg = '0') THEN -- When clock goes back to zero, reset zero flag
--        clk_zero <= '0';
--      END IF;
--   END PROCESS;
--   
--   PROCESS (data_in_del, rst, clk_zero, data_in )
--   BEGIN
--      IF (rst = '1') THEN
--        clk_out_reg <= '0';
--      ELSIF (data_in_del'EVENT AND (data_in_del = '1' OR data_in_del = '0')) THEN
--         IF (rx_ena = '1') THEN 
--           clk_out_reg <= data_in_del xor invert; -- Phase is changed for each zero
--         END IF;
--      ELSIF ((data_in'EVENT AND (data_in = '1' OR data_in = '0')) and clk_zero = '1') THEN -- For resetting of clock after a zero
--        clk_out_reg <= '0' ;
--      END IF;
--   END PROCESS;
--   
--   data_in_xor_del <= data_in_del xor data_in;
--   data_out <= data_out_reg;
--   clk_out <= clk_out_reg;
   
--   END ARCHITECTURE logical;