---------------------------------------------------------------------------------------------------
-- DESIGN RCU FIRM WARE V.2.0
-- FILE NAME   : RCU_REG.VHD
-- DESIGN TEAM : Luciano Musa, Changzhou Xiang , Attiq ur Rehman
-- DESCRIPTION : All control Signal Active High, Data RD/WR on rising edge,ASYNCHRONOUS RESET AND CLEAR
-- CLOCK DOMAIN: RCU CLOCK
-- PURPOSE     : RCU STAUS REGISTER AND CONFIGURATION REGISTER
--           
-- AUTHORS     : Changzhou.Xiang (CERN-PH/ED)
--
-- UPDATE TIME : 21.07.08
-- NOTE        : Add FSM status of DA.
--
-- UPDATE TIME : 12.06.08
-- NOTE        : Add "inst_err_chk" as ouput, instruction for TPC/PHOS/FMD.
--
---- UPDATE TIME : 10.06.08
-- NOTE        : 		FEC_AL_fsc 	we_FEC_AL_fsc added to write the fec active list from safty module.
-- UPDATE TIME : 29.05.08
-- NOTE        : Add "da_bsy" as input.
--
-- UPDATE TIME : 28.05.08
-- NOTE        : Add checking RDYRX status register.
--             : Change scel_err_reg width from 8 to 6.
--             : Change dstb/trsf counters bit width.
--
-- UPDATE TIME : 23.05.08
-- NOTE        : Modify RCU bus busy register from 4 bits to 1bit.
--
-- UPDATE TIME : 19.05.08
-- NOTE        : Add meb status counter to registers.
--
-- UPDATE TIME : 18.05.087
-- NOTE        : Add RCU bus busy status register.
--
-- UPDATE TIME : 16.05.08
-- NOTE        : Add RCU ID register.
--
-- UPDATE TIME : 15.05.08
-- NOTE        : Change dstb/trsf/ack/cstb counters bit width.
--
-- UPDATE TIME : 12.05.08
-- NOTE        : Change "scel_err_reg" bit width to 8bit.
--
-- UPDATE TIME : 11.05.08
-- NOTE        : Add enable/disable configuration bit for check RDYRX status in register RDOMOD[3].
--               Default value is 0, enable checking. When set 1, disable checking.
--               Add ALTRO bus signal(dstb/trsf/cstb/ack) counters.
--               Add block length of last written data memory.
--               Set register address width as 8bit.
-------------------------------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


library WORK;
use work.rcu_addr_set.all;

entity RCU_REG is
    Port(

		en 					: in  STD_LOGIC; 	-- ENABLE
		we 					: in  STD_LOGIC;	-- WRITE ENABLE
		addr					: in  STD_LOGIC_VECTOR(7 downto 0);		-- RCU ADDRESS
		din 					: in  STD_LOGIC_VECTOR(31 downto 0);	-- RCU BUS DATA-IN
		altro_err_a			: in  STD_LOGIC;
		altro_err_b			: in  STD_LOGIC;
		
		fecerr_a				: in  STD_LOGIC_VECTOR(31 downto 0);	-- FEC ERROR REGISTER FOR BRANCH A
		fecerr_b				: in  STD_LOGIC_VECTOR(31 downto 0);	-- FEC ERROR REGISTER FOR BRANCH B
		da_bsy	 			: in  std_logic;	-- DA IS BUSY
		rd_rx					: in  STD_LOGIC;	-- RDYRX from SIU interface
		clr_rdrx_reg		: in  STD_LOGIC;	-- CLEAR READY RECEIVE STATUS
		scel_err_reg		: in  STD_LOGIC_VECTOR(5 downto 0); 	-- SCAN EVENT LENGTH REGISTER
		blk_err_reg 		: in  STD_LOGIC;	-- DATA MEMORY BLOCK LENGTH ERROR REGISTER
		add_err_reg 		: in  STD_LOGIC;	-- DATA MEMORY ADDRESS MISSMATCH ERROR REGISTER
		rcu_busbsy			: in  STD_LOGIC_VECTOR(1 downto 0);		-- INS_SEQ RUNING STATUS AND GETTING FAILED INSTUCTION REGISTER
		meb_st_cnt			: in  STD_LOGIC_VECTOR(3 downto 0);		-- MEB STATUS COUNTER
		swt_trg_cnt			: in  STD_LOGIC_VECTOR(19 downto 0);	-- NUMBER OF SOFTWARE TRIGER
		aux_trg_cnt			: in  STD_LOGIC_VECTOR(19 downto 0);	-- NUMBER OF AUX TRIGER
		ttc_L2a_cnt			: in  STD_LOGIC_VECTOR(19 downto 0);	-- NUMBER OF SOFTWARE TRIGER
		ttc_L2r_cnt			: in  STD_LOGIC_VECTOR(19 downto 0);	-- NUMBER OF L2r TRIGER
		ttc_L1r_cnt			: in  STD_LOGIC_VECTOR(19 downto 0);	-- NUMBER OF L1r TRIGER
		dout 					: out STD_LOGIC_VECTOR(31 downto 0);	-- DATA OUT TO RCU BUS
		
		
		fec_act_list		: out STD_LOGIC_VECTOR(31 downto 0);	-- ACTIVE FEC LIST
		inst_err_chk		: out STD_LOGIC_VECTOR( 1 downto 0);	-- Instruction error checking selection.
		prg_dly				: out STD_LOGIC_VECTOR( 1 downto 0);	-- PROGRAMABLE DELAY BETWEEN "WRITE" AND "CSTB" OF ALTRO
		samp_frec			: out STD_LOGIC_VECTOR(3 downto 0);		-- SAMPLE FREQUENCE.
		
		trg_source			: out STD_LOGIC_VECTOR( 2 downto 0);	-- TRIGER SOURCE
		trg_mode				: out STD_LOGIC;	-- TRIGER MODE. PHOS OR TPC
		trg_prg_dly			: out STD_LOGIC_VECTOR(12 downto 0);	-- PROGRAMABLE DELAY BETWEEN "L1" AND "L2" OF TEST TRIGER
		en_chkrdyrx_n		: out STD_LOGIC;	-- ENABLE/DISABLE CHECK RDYRX
		sprs_rdo				: out STD_LOGIC;	-- SPARSE READOUT STATUS
		skp_emp_ch				: out STD_LOGIC;
		es_on					: out STD_LOGIC;	-- EXCUTE SEQUENCER ON SOE/EOE
		meb_mode				: out STD_LOGIC;	-- MEB MODE. 4 OR 8 WITH LENGTH
		rdo_cfg1				: out STD_LOGIC_VECTOR(19 downto 0);	-- ALTRO CONFIGURE REGISTER1
		rdo_cfg2				: out STD_LOGIC_VECTOR(24 downto 5);	-- ALTRO CONFIGURE REGISTER2
		rcu_version 		: out STD_LOGIC_VECTOR(31 downto 0);	-- RCU version
		rcu_id 				: out STD_LOGIC_VECTOR( 8 downto 0);	-- RCU ID
		bp_version  		: out STD_LOGIC;	-- BP version
		
-- mask the check on block length mismatch and channel addres mismatch.

		mask_ch_add 		: out STD_LOGIC;
		mask_blk_len 		: out STD_LOGIC;
		
-- altro bus signals are added to monitor the status

		cstb_a 				: in  STD_LOGIC;
		ack_a 				: in  STD_LOGIC;
		trsf_a 				: in  STD_LOGIC;
		cstb_b 				: in  STD_LOGIC;
		ack_b 				: in  STD_LOGIC;
		trsf_b 				: in  STD_LOGIC;
		L1 					: in  STD_LOGIC;
		L2 					: in  STD_LOGIC;
		
--  safty module write signals for the active fron end card list

		FEC_AL_fsc 			: IN std_logic_vector(31 downto 0);
		we_FEC_AL_fsc 		: IN std_logic;
		
		-- ALTRO bus signal counters
		dstba_cnt			: in  std_logic_vector(19 downto 0);	-- number of dstb negedge for branch a  
		dstbb_cnt			: in  std_logic_vector(19 downto 0);	-- number of dstb negedge for branch b
		trsfa_cnt			: in  std_logic_vector(11 downto 0);	-- number of trsf posedge for branch a  
		trsfb_cnt			: in  std_logic_vector(11 downto 0);	-- number of trsf posedge for branch b 
		acka_cnt				: in  std_logic_vector(11 downto 0);	-- number of ack posedge for branch a  
		ackb_cnt				: in  std_logic_vector(11 downto 0);	-- number of ack posedge for branch b 
		cstba_cnt			: in  std_logic_vector(11 downto 0);	-- number of cstb posedge for branch a  
		cstbb_cnt			: in  std_logic_vector(11 downto 0);	-- number of cstb posedge for branch b  
		
		dstb_numA			: in  std_logic_vector(8 downto 0);		-- DSTB NUMBER OF LAST BLOCK FOR BRANCH A
		dstb_numB			: in  std_logic_vector(8 downto 0);		-- DSTB NUMBER OF LAST BLOCK FOR BRANCH B
		bl_err_cnt			: in  STD_LOGIC_VECTOR(11 downto 0);		-- Block length mismatch counter
		chadd_err_cnt		: in STD_LOGIC_VECTOR(11 downto 0);
		rdrx_erreg			: out std_logic;	-- RDYRX error register
		rdo_err				: out std_logic_vector(8 downto 0);	
		
		
		fsmwr_st_a			: in  STD_LOGIC_VECTOR(3 downto 0);		-- FSM state,from AIM 
		fsmrd_st_a			: in  STD_LOGIC_VECTOR(3 downto 0);		-- FSM state,from AIM 
		fsmwr_st_b			: in  STD_LOGIC_VECTOR(3 downto 0);		-- FSM state,from AIM 
		fsmrd_st_b			: in  STD_LOGIC_VECTOR(3 downto 0);		-- FSM state,from AIM 
		fsmevrdoctrl_st		: in  STD_LOGIC_VECTOR(3 downto 0);		-- FSM state,from AIM 
		fsmselbr_st_a		: in  STD_LOGIC_VECTOR(1 downto 0);		-- FSM state,from AIM 
		fsmselbr_st_b		: in  STD_LOGIC_VECTOR(1 downto 0);		-- FSM state,from AIM 
		fsmevlen_st			: in  STD_LOGIC_VECTOR(3 downto 0);		-- FSM state,from AIM 
		fsmscevlen_st		: in  STD_LOGIC_VECTOR(2 downto 0);		-- FSM state,from AIM 
		fsmrdo_st_a			: in  STD_LOGIC_VECTOR(2 downto 0);		-- FSM state,from AIM 
		fsmrdo_st_b			: in  STD_LOGIC_VECTOR(2 downto 0);		-- FSM state,from AIM 
		fsminstseq_st		: in  STD_LOGIC_VECTOR(4 downto 0);		-- FSM state,from IS
		fsminstloop_st		: in  STD_LOGIC_VECTOR(4 downto 0);		-- FSM state,from IS 
		fsmtsttrg_st		: in  STD_LOGIC_VECTOR(1 downto 0);		-- FSM state,from EV_MNG
		fsmttctrg_st		: in  STD_LOGIC_VECTOR(3 downto 0);		-- FSM state,from EV_MNG
		fsmev_st			   : in  STD_LOGIC_VECTOR(2 downto 0);		-- FSM state,from EV_MNG
		fsminsseq_st		: in  STD_LOGIC_VECTOR(2 downto 0);		-- FSM state,from EV_MNG 	
		fsmru_st			   : in  STD_LOGIC_VECTOR(4 downto 0);		-- FSM state,from RU	
		fsmdamain_st		: in  STD_LOGIC_VECTOR(4 downto 0);		-- FSM state,from DA 	
		fsmrdcdh_st			: in  STD_LOGIC_VECTOR(4 downto 0);		-- FSM state,from DA 	
		fsmrdcdhsoft_st	: in  STD_LOGIC_VECTOR(4 downto 0);		-- FSM state,from DA 	
		fsmrddm_st			: in  STD_LOGIC_VECTOR(4 downto 0);		-- FSM state,from DA 	
		fsmrdpar_st			: in  STD_LOGIC_VECTOR(4 downto 0);		-- FSM state,from DA 	
		fsmrdrm_st			: in  STD_LOGIC_VECTOR(3 downto 0);		-- FSM state,from DA 
		
		data_ready			: in  STD_LOGIC_VECTOR(3 downto 0);	-- status of data memories from AIM
		dm_sel				: in  STD_LOGIC_VECTOR(1 downto 0) ; -- from data assembler
		
		meb_full 			: in  STD_LOGIC; -- debug for Busy Box and MEB
		clk 		: in  STD_LOGIC;
		rst			: in  STD_LOGIC
	);
end RCU_REG;

architecture A_RCUREG of RCU_REG is    	
										-- change from 220909AE to fix the MEB Status counter	
	constant reg_RCUversion		: std_logic_vector(31 downto 0):= X"231109AE";

	signal reg_fecactlist		: std_logic_vector(31 downto 0) :=(others=>'0');
	signal reg_altroif			: std_logic_vector(17 downto 0) := "00"&X"0400";
	signal reg_trgconf			: std_logic_vector(16 downto 0) := "0" & X"0000";
	signal reg_rdomod			 	: std_logic_vector( 6 downto 0) := "000"& X"0";
	signal reg_rdocfg1			: std_logic_vector(19 downto 0) :=(others=>'0');
	signal reg_rdocfg2			: std_logic_vector( 6 downto 0) :=(others=>'0');
	signal reg_rcuid				: std_logic_vector( 8 downto 0) :="000000000";
	signal reg_BPversion			: std_logic := '0';
	signal rdrx_reg				: std_logic := '0';	-- RDYRX error register
	signal st_meb_full			: std_logic := '0';
	signal bus_stat ,bus_st_trsf	: std_logic_vector(7 downto 0);
	signal trsf_trg					: std_logic;

begin

	rdrx_erreg <= rdrx_reg;
	
	latch_meb_full :	process(meb_full,clk)
	begin
		if meb_full='1' then
				st_meb_full <= '1';
		end if;
	end process;
	
	process(rst,clr_rdrx_reg,clk,reg_rdomod(3))
	begin
		if rst='1' or clr_rdrx_reg='1' or reg_rdomod(3)='1' then
			rdrx_reg <= '0';
		elsif clk' event and clk='1' then
			if rd_rx='0' and da_bsy='1' then
				rdrx_reg <= '1';
			end if;
		end if;
	end process;
	
		trsf_trg <= trsf_a or trsf_b ;
	
	process(rst,clk)
		begin
			if rst='1' then		
				bus_stat <= (others=>'0');  
					
					elsif clk' event and clk='1' then
					
					bus_stat(0) <= cstb_a;
					bus_stat(1) <= ack_a;
					bus_stat(2) <= trsf_a;
					bus_stat(3) <= cstb_b;
					bus_stat(4) <= ack_b;
					bus_stat(5) <= trsf_b;
					bus_stat(6) <= altro_err_a;
					bus_stat(7) <= altro_err_b;
					
			end if; 
	end process;

	process(rst,clk)
		begin
			if rst='1' then		
				bus_st_trsf <= (others=>'0');  
					
				elsif clk' event and clk='1' then
					
					if trsf_trg = '1' then
					
					bus_st_trsf(0) <= cstb_a;
					bus_st_trsf(1) <= ack_a;
					bus_st_trsf(2) <= trsf_a;
					bus_st_trsf(3) <= cstb_b;
					bus_st_trsf(4) <= ack_b;
					bus_st_trsf(5) <= trsf_b;
					bus_st_trsf(6) <= L1;
					bus_st_trsf(7) <= L2;
					
					end if;
					
			end if; 
	end process;
		
	process(rst,clk)
	begin
		if clk' event and clk='1' then
			if en='1' and we='1' then
				case addr is        
					when altroif	=>
						reg_altroif <= din(17 downto 0);

					when trgconf	=>
						reg_trgconf <= din(16 downto 0);
						
					when rdomod	=>
						reg_rdomod <= din(6 downto 0);
						
					when rdocfg1 	=>
						reg_rdocfg1 <= din(19 downto 0);
						
					when rdocfg2 	=>
						reg_rdocfg2 <= din(6 downto 0);
					when rcuid 	=>
						reg_rcuid <= din(8 downto 0);					
					when bpversion	=>
						reg_bpversion <= din(0);

					when others=>
				end case;
			end if;
		end if;
	end process;
	
	
	ACTFECLIST : process(clk)
	begin
		if clk' event and clk='1' then
			if en='1' and we='1' and (addr = fecactlist) then
				reg_fecactlist <= din;
				elsif we_FEC_AL_fsc = '1' then
				 reg_fecactlist <= FEC_AL_fsc;
			end if;

		end if;
	end process;
	
	process(clk,rst)
	begin
		if rst='1' then
			dout <= (others=>'0');
		elsif clk' event and clk='1' then
			if en='1' and we='0' then			
				case addr is   
					-- Configuration register
					when fecactlist =>
						dout <= reg_fecactlist;
                                            
					when altroif =>
						dout <= X"000"&"00"&reg_altroif;
						                       
					when trgconf =>
						dout <= X"000" & "000" &reg_trgconf;

					when rdomod =>
						dout <= X"000000"&"0"&reg_rdomod;

					when rdocfg1 	=>
						dout <= X"000"&reg_rdocfg1;	-- 20 bit
						
					when rdocfg2 	=>
						dout <= X"000000"&"0"&reg_rdocfg2; -- 7 bit 
						
					when rcuversion =>
						dout <= reg_rcuversion;			-- 24 bit

					when bpversion =>
						dout <= X"0000000"&"000"&reg_BPversion;		-- 1 bit
						
					when rcuid =>
						dout <= X"00000"&"000"&reg_rcuid;		-- 9 bit
					
					-- Status&Error register
					when fecerra 	=>					
						dout <= fecerr_a;
						
					when fecerrb 	=> 
						dout <= fecerr_b;

					when rdoerr 	=> 
						dout <= fsmwr_st_b & fsmwr_st_a & fsmevlen_st & X"00"&"000" &blk_err_reg&add_err_reg&rdrx_reg&scel_err_reg;

					when busstat =>
						dout <= X"000000"& bus_stat;  	-- (4*6=24)+8 bit

					when bussttrsf 	=>
						dout <= X"000000"& bus_st_trsf;  	-- (4*6=24)+8 bit
						
					when rcubusbsy 	=> 
						dout <= X"0000000" & "00" & rcu_busbsy;
						
					when mebstcnt =>
						dout <= X"00000" & '0' & dm_sel & data_ready & st_meb_full & meb_st_cnt;
						
					when swttrgcnt =>
						dout <= "000000000000"&swt_trg_cnt;
						
					when auxtrgcnt =>
						dout <= "000000000000"&aux_trg_cnt;
						
					when ttcL2acnt =>
						dout <= "000000000000"&ttc_L2a_cnt;
						
					when ttcL2rcnt =>
						dout <= "000000000000"&ttc_L2r_cnt;
					when ttcL1rcnt =>
						dout <= "000000000000"&ttc_L1r_cnt;
						
					when dstbacnt 	=>	
						dout <= X"000"& dstba_cnt;
						
					when dstbbcnt 	=>	
						dout <= X"000"& dstbb_cnt;
						
					when trsfacnt 	=>	
						dout <= X"00000"& trsfa_cnt;
						
					when trsfbcnt 	=>	
						dout <= X"00000"& trsfb_cnt;
						
					when ackacnt 	=>	
						dout <= X"00000"& acka_cnt;
						
					when ackbcnt 	=>	
						dout <= X"00000"& ackb_cnt;
							
					when cstbacnt 	=>	
						dout <= X"00000"& cstba_cnt;
						
					when cstbbcnt 	=>	
						dout <= X"00000"& cstbb_cnt;
						
					when dstbnuma 	=>	
						dout <= X"00000"&"000"& dstb_numA;
												
					when dstbnumb 	=>	
						dout <= X"00000"&"000"& dstb_numB;  
						
					when chadderrcnt 	=>	
						dout <= X"00000"&chadd_err_cnt;												
						
					when blerrcnt 	=>	
						dout <= X"00000"&bl_err_cnt;               		 
					
					when abdfsm_st	=> 
					
					dout <= X"00"&"00"&fsmrdo_st_b&fsmrd_st_b&fsmwr_st_b&fsmrdo_st_a&fsmrd_st_a&fsmwr_st_a;				

					when rdofsm_st	=> 
						dout <= X"0000"&"0"&fsmscevlen_st&fsmevlen_st&fsmevrdoctrl_st&fsmselbr_st_b&fsmselbr_st_a;
						
					when insseq_st	=> 
						dout <= X"0000"&"0"&fsminstseq_st&fsminstloop_st&fsmru_st;
						
					when evmng_st	=>
						dout <= X"00000"&fsmtsttrg_st&fsmttctrg_st&fsmev_st&fsminsseq_st;    

					when da_st	=>
						dout <= "000" & fsmrdrm_st & fsmrdpar_st & fsmrddm_st & fsmrdcdhsoft_st & fsmrdcdh_st & fsmdamain_st;
						
					when others=>
						dout <= (others=>'0');
				end case;
			end if;
		end if;
	end process;
	
	fec_act_list<= reg_fecactlist;
	
	inst_err_chk<= reg_altroif(17 downto 16);
	prg_dly		<= reg_altroif(15 downto 14);
	samp_frec	<= reg_altroif(13 downto 10);
	
	trg_source	<= reg_trgconf(16 downto 14);
	trg_mode	<= reg_trgconf(13);
	trg_prg_dly	<= reg_trgconf(12 downto 0);
	
	skp_emp_ch	   <= reg_rdomod(6);
	mask_ch_add    <= reg_rdomod(5);
	mask_blk_len   <= reg_rdomod(4);			
	en_chkrdyrx_n	<= reg_rdomod(3);
	sprs_rdo			<= reg_rdomod(2);
	es_on				<= reg_rdomod(1);
	meb_mode			<= reg_rdomod(0);
 
	
	-- PARAMETER RDOCONFIG1/2 FOR TRAILER WORD IS CONSTRUCTED HERE TO PASS THE DATA ASSEMBLER.	
	rdo_cfg1				<= reg_rdocfg1;
	rdo_cfg2(8 downto 5)	<= reg_altroif(13 downto 10);	-- samp_frec
	rdo_cfg2(9)				<= reg_rdomod(2);	-- sprs_rdo
	rdo_cfg2(19 downto 10)	<= reg_altroif(9 downto 0);	--nsamples
	rdo_cfg2(23 downto 20)	<= reg_rdocfg2(3 downto 0); -- PTRG, from ALTRO DPCF2
	rdo_cfg2(24)			<= reg_rdocfg2(4); -- NBUF, from ALTRO DPCF2
	
	rdo_err <= 	blk_err_reg&add_err_reg&rdrx_reg&scel_err_reg;
	rcu_version <=reg_rcuversion;
	rcu_id <=reg_rcuid;
	bp_version <=reg_bpversion;
	
end;

