-------------------------------------------------------------------------------
-- Title      : Trigger Receiver Package
-- Project    : RCU Trigger Receiver
-------------------------------------------------------------------------------
-- File             : $RCSfile: trigger_receiver_pkg.vhd,v $
-- Last edited by   : $Author: alme $
-- Last update      : $Date: 2010/03/24 08:32:48 $
-- Current Revision : $Revision: 1.12 $
-------------------------------------------------------------------------------
-- Description      : Constant and function library for RCU Trigger Receiver
--                    design  
-------------------------------------------------------------------------------
-- Revision History : 
-- http://portal1.ift.uib.no/cgi-bin/viewcvs.cgi/vhdlcvs/rcu_cpld/
-------------------------------------------------------------------------------
--
-- This file is property of and copyright by the Instrumentation and 
-- Electronics Section, Dep. of Physics and Technology
-- University of Bergen, Norway, 2005
-- This file has been written by Johan Alme
-- Johan.Alme@ift.uib.no
--
-- Permission to use, copy, modify and distribute this firmware and its  
-- documentation strictly for non-commercial purposes is hereby granted  
-- without fee, provided that the above copyright notice appears in all  
-- copies and that both the copyright notice and this permission notice  
-- appear in the supporting documentation. The authors make no claims    
-- about the suitability of this software for any purpose. It is         
-- provided "as is" without express or implied warranty.                 
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

package trigger_receiver_pkg is
  
  -- Generates a hamming code for a 8 bit word
  function hamming8(data : in std_logic_vector(7 downto 0) )
                    return std_logic_vector;

  -- Decodes a hamming protected 8 bit word
  function hamming8(data : in std_logic_vector(7 downto 0);
                    hmg_code: in std_logic_vector(4 downto 0)) --received hamming
                    return std_logic_vector;

  -- Generates a hamming code for a 32 bit word
  function hamming32(data : in std_logic_vector(31 downto 0))
                     return std_logic_vector;

  -- Decodes a hamming protected 32 bit word
  function hamming32(data : in std_logic_vector(31 downto 0);
                     hmg_code: in std_logic_vector(6 downto 0)) --received hamming
                     return std_logic_vector;
  

  -- Generates a parity bit of a 32 bit word
  function parity32(data : in std_logic_vector(31 downto 0)) return std_logic;

    -- defining types for the addressed messages
  type message_array     is array(integer range <>) of std_logic_vector(11 downto 0);
  type DAQ_header_array  is array(integer range <>) of std_logic_vector(31 downto 0);

  constant c_Module_address   : std_logic_vector(3 downto 0)  :=X"4";
  -- RCU register space addresses
  constant c_control          : std_logic_vector(11 downto 0) := X"000";
  constant c_module_reset     : std_logic_vector(11 downto 0) := X"001";
  constant c_reset_counters   : std_logic_vector(11 downto 0) := X"004";
  constant c_testmode         : std_logic_vector(11 downto 0) := X"005";
  constant c_L1_Latency       : std_logic_vector(11 downto 0) := X"006";
  constant c_L2_Latency       : std_logic_vector(11 downto 0) := X"007";
  constant c_PrePulse_Latency : std_logic_vector(11 downto 0) := X"008";
  constant c_L2_ExtendedLatency : std_logic_vector(11 downto 0) := X"009";
  constant c_L1_msg_Latency   : std_logic_vector(11 downto 0) := X"00A";
  constant c_pp_counter       : std_logic_vector(11 downto 0) := X"00B";
  constant c_BCID_local       : std_logic_vector(11 downto 0) := X"00C";
  constant c_L0_counter       : std_logic_vector(11 downto 0) := X"00D";
  constant c_L1_counter       : std_logic_vector(11 downto 0) := X"00E";
  constant c_L1_msg_counter   : std_logic_vector(11 downto 0) := X"00F";
  constant c_L2a_counter      : std_logic_vector(11 downto 0) := X"010";
  constant c_L2r_counter      : std_logic_vector(11 downto 0) := X"011";
  constant c_hammingerrorcnt  : std_logic_vector(11 downto 0) := X"016";
  constant c_errorcnt         : std_logic_vector(11 downto 0) := X"017";
  constant c_buffered_events  : std_logic_vector(11 downto 0) := X"020";
  constant c_DAQ_header01     : std_logic_vector(11 downto 0) := X"021";
  constant c_DAQ_header02     : std_logic_vector(11 downto 0) := X"022";
  constant c_DAQ_header03     : std_logic_vector(11 downto 0) := X"023";
  constant c_DAQ_header04     : std_logic_vector(11 downto 0) := X"024";
  constant c_DAQ_header05     : std_logic_vector(11 downto 0) := X"025";
  constant c_DAQ_header06     : std_logic_vector(11 downto 0) := X"026";
  constant c_DAQ_header07     : std_logic_vector(11 downto 0) := X"027";
  constant c_event_info       : std_logic_vector(11 downto 0) := X"028";
  constant c_event_error      : std_logic_vector(11 downto 0) := X"029";
  -- addresses used only in debug mode
  constant c_bunchcounter     : std_logic_vector(11 downto 0) := X"013";
  constant c_L1messageheader  : std_logic_vector(11 downto 0) := X"030";
  constant c_L1messagedata1   : std_logic_vector(11 downto 0) := X"031";
  constant c_L1messagedata2   : std_logic_vector(11 downto 0) := X"032";
  constant c_L1messagedata3   : std_logic_vector(11 downto 0) := X"033";
  constant c_L1messagedata4   : std_logic_vector(11 downto 0) := X"034";
  constant c_L2amessageheader : std_logic_vector(11 downto 0) := X"035";
  constant c_L2amessagedata1  : std_logic_vector(11 downto 0) := X"036";
  constant c_L2amessagedata2  : std_logic_vector(11 downto 0) := X"037";
  constant c_L2amessagedata3  : std_logic_vector(11 downto 0) := X"038";
  constant c_L2amessagedata4  : std_logic_vector(11 downto 0) := X"039";
  constant c_L2amessagedata5  : std_logic_vector(11 downto 0) := X"03A";
  constant c_L2amessagedata6  : std_logic_vector(11 downto 0) := X"03B";
  constant c_L2amessagedata7  : std_logic_vector(11 downto 0) := X"03C";
  constant c_L2rmessageheader : std_logic_vector(11 downto 0) := X"03D";
  constant c_TESTread_enable  : std_logic_vector(11 downto 0) := X"080";
  constant c_TESTheader       : std_logic_vector(11 downto 0) := X"081";


  -- broadcast message decoding:
  --constant c_usermsg_pre_pulse   : std_logic_vector(1 downto 0) := "11";
  --constant c_systemmsg_pre_pulse : std_logic_vector(3 downto 0) := "1111";

  -- individual address decoding:
  constant c_TTCrx_addr  : std_logic_vector(15 downto 0) := X"0003";-- the first
  constant c_L1_header   : std_logic_vector(3 downto 0)  := X"1";
  constant c_L1_data     : std_logic_vector(3 downto 0)  := X"2";
  constant c_L2a_header  : std_logic_vector(3 downto 0)  := X"3";
  constant c_L2a_data    : std_logic_vector(3 downto 0)  := X"4";
  constant c_L2r_header  : std_logic_vector(3 downto 0)  := X"5";
  constant c_FEE_reset   : std_logic_vector(3 downto 0)  := X"8";
  
  -- DAQ header version
  constant c_headerversion : std_logic_vector := X"2";
  -- trigger receiver version
  constant c_version : std_logic_vector := X"17"; --version 1.7
  -- default values
  constant df_L1_latency        : std_logic_vector := X"0D4";
  constant df_L1_boundary       : std_logic_vector := X"2";
  constant df_L1msg_latency_min : std_logic_vector := X"0028";
  constant df_L1msg_latency_max : std_logic_vector := X"4E20";
  constant df_L2msg_latency_min : std_logic_vector := X"0C80";
  constant df_L2msg_latency_max : std_logic_vector := X"4E20";
  constant df_L2_ExtendedLatency_min : std_logic_vector := X"0C80";
  constant df_L2_ExtendedLatency_max : std_logic_vector := X"4E48";--500 us + 1 us
         
end package trigger_receiver_pkg;

package body trigger_receiver_pkg is
  
  --purpose: hamming decoding 8 bit
  function hamming8 (data : in std_logic_vector(7 downto 0) ) 
    return std_logic_vector is
    variable hmg: std_logic_vector(4 downto 0);
  begin
    hmg(0) := data(0) xor data(1) xor data(2) xor data(3);
    hmg(1) := data(0) xor data(4) xor data(5) xor data(6);
    hmg(2) := data(1) xor data(2) xor data(4) xor data(5) xor data(7);
    hmg(3) := data(1) xor data(3) xor data(4) xor data(6) xor data(7); 
    hmg(4) := hmg(0) xor hmg(1) xor hmg(2) xor hmg(3) xor data(0) xor data(1) xor data(2) xor 
              data(3) xor data(4) xor data(5) xor data(6) xor data(7); 
    return hmg;
  end function hamming8;
  
  function hamming8 (data : in std_logic_vector(7 downto 0);
                     hmg_code: in std_logic_vector(4 downto 0) ) 
    return std_logic_vector is
    variable hmg: std_logic_vector(4 downto 0);
  begin
    hmg(0) := data(0) xor data(1) xor data(2) xor data(3);
    hmg(1) := data(0) xor data(4) xor data(5) xor data(6);
    hmg(2) := data(1) xor data(2) xor data(4) xor data(5) xor data(7);
    hmg(3) := data(1) xor data(3) xor data(4) xor data(6) xor data(7); 
    hmg(4) := hmg_code(0) xor hmg_code(1) xor hmg_code(2) xor hmg_code(3) xor data(0) xor data(1) xor data(2) xor 
              data(3) xor data(4) xor data(5) xor data(6) xor data(7); 
    return hmg;
  end function hamming8;
  
  function hamming32(data : in std_logic_vector(31 downto 0)) 
    return std_logic_vector is
    variable hmg: std_logic_vector(6 downto 0);
  begin
    hmg(0) := data(0) xor data(1) xor data(2) xor data(3) xor data(4) xor data(5);
    hmg(1) := data(6) xor data(7) xor data(8) xor data(9) xor data(10) xor data(11) xor data(12) xor data(13) xor 
              data(14) xor data(15) xor data(16) xor data(17) xor data(18) xor data(19) xor data(20);
    hmg(2) := data(6) xor data(7) xor data(8) xor data(9) xor data(10) xor data(11) xor data(12) xor 
              data(13) xor data(21) xor data(22) xor data(23) xor data(24) xor data(25) xor data(26) xor data(27);
    hmg(3) := data(0) xor data(1) xor data(2) xor data(6) xor data(7) xor data(8) xor data(9) xor 
              data(14) xor data(15) xor data(16) xor data(17) xor data(21) xor data(22) xor data(23) xor 
              data(24) xor data(28) xor data(29) xor data(30);
    hmg(4) := data(0) xor data(3) xor data(4) xor data(6) xor data(7) xor data(10) xor data(11) xor data(14) xor 
              data(15) xor data(18) xor data(19) xor data(21) xor data(22) xor data(25) xor data(26) xor data(28) xor 
              data(29) xor data(31);
    hmg(5) := data(1) xor data(3) xor data(5) xor data(6) xor data(8) xor data(10) xor data(12) xor data(14) xor
              data(16) xor data(18) xor data(20) xor data(21) xor data(23) xor data(25) xor data(27) xor 
              data(28) xor data(30) xor data(31);
    hmg(6) := hmg(0) xor hmg(1) xor hmg(2) xor hmg(3) xor hmg(4) xor hmg(5) xor 
              data(0) xor data(1) xor data(2) xor data(3) xor data(4) xor data(5) xor data(6) xor data(7) xor 
              data(8) xor data(9) xor data(10) xor data(11) xor data(12) xor data(13) xor data(14) xor 
              data(15) xor data(16) xor data(17) xor data(18) xor data(19) xor data(20) xor data(21) xor 
              data(22) xor data(23) xor data(24) xor data(25) xor data(26) xor data(27) xor data(28) xor 
              data(29) xor data(30) xor data(31);
    return hmg;
  end function hamming32;

  function hamming32(data    : in std_logic_vector(31 downto 0);
                     hmg_code: in std_logic_vector(6 downto 0)) --received hamming
    return std_logic_vector is
    variable hmg: std_logic_vector(6 downto 0);
  begin
    hmg(0) := data(0) xor data(1) xor data(2) xor data(3) xor data(4) xor data(5);
    hmg(1) := data(6) xor data(7) xor data(8) xor data(9) xor data(10) xor data(11) xor data(12) xor data(13) xor 
              data(14) xor data(15) xor data(16) xor data(17) xor data(18) xor data(19) xor data(20);
    hmg(2) := data(6) xor data(7) xor data(8) xor data(9) xor data(10) xor data(11) xor data(12) xor 
              data(13) xor data(21) xor data(22) xor data(23) xor data(24) xor data(25) xor data(26) xor data(27);
    hmg(3) := data(0) xor data(1) xor data(2) xor data(6) xor data(7) xor data(8) xor data(9) xor 
              data(14) xor data(15) xor data(16) xor data(17) xor data(21) xor data(22) xor data(23) xor 
              data(24) xor data(28) xor data(29) xor data(30);
    hmg(4) := data(0) xor data(3) xor data(4) xor data(6) xor data(7) xor data(10) xor data(11) xor data(14) xor 
              data(15) xor data(18) xor data(19) xor data(21) xor data(22) xor data(25) xor data(26) xor data(28) xor 
              data(29) xor data(31);
    hmg(5) := data(1) xor data(3) xor data(5) xor data(6) xor data(8) xor data(10) xor data(12) xor data(14) xor 
              data(16) xor data(18) xor data(20) xor data(21) xor data(23) xor data(25) xor data(27) xor 
              data(28) xor data(30) xor data(31);
    hmg(6) := hmg_code(0) xor hmg_code(1) xor hmg_code(2) xor hmg_code(3) xor hmg_code(4) xor hmg_code(5) xor 
              data(0) xor data(1) xor data(2) xor data(3) xor data(4) xor data(5) xor data(6) xor data(7) xor 
              data(8) xor data(9) xor data(10) xor data(11) xor data(12) xor data(13) xor data(14) xor 
              data(15) xor data(16) xor data(17) xor data(18) xor data(19) xor data(20) xor data(21) xor 
              data(22) xor data(23) xor data(24) xor data(25) xor data(26) xor data(27) xor data(28) xor 
              data(29) xor data(30) xor data(31);
    return hmg;
  end function hamming32;

  -- calculates the parity of a 32 bit word.
  function parity32(data : in std_logic_vector(31 downto 0))
    return std_logic is
    variable parity_bit : std_logic;
  begin
    parity_bit := data(0) xor data(1) xor data(2) xor data(3) xor data(4) xor data(5) xor data(6) xor data(7) xor 
                  data(8) xor data(9) xor data(10) xor data(11) xor data(12) xor data(13) xor data(14) xor 
                  data(15) xor data(16) xor data(17) xor data(18) xor data(19) xor data(20) xor data(21) xor 
                  data(22) xor data(23) xor data(24) xor data(25) xor data(26) xor data(27) xor data(28) xor 
                  data(29) xor data(30) xor data(31);
    return parity_bit;
  end function parity32;
  -- specific ROI check for TPC. (Can be modified for other sub-detectors
  --function RoI_check(
  --  RoI        : in std_logic_vector(35 downto 0); 
  --  RoI_config : in std_logic_vector(35 downto 0)) 
  --return std_logic is
  --  variable result : std_logic;  
  --begin
    -- testing if the RoI config register and the received RoI both are '1'.
    -- If so the RCU is within the Region of Interest.
  --  result := '0';
  --  for i in 0 to RoI_Config'length-1 loop
  --    if (RoI(i) = '1' and RoI_Config(i) = '1') then
  --      result := '1';
  --    end if;          
  --  end loop;
    
  --  return result;
  --end function;
  
end package body trigger_receiver_pkg;
